// xform - A class that does 2D transformations: translation, rotation & zoom
// by Thomas Kisko

#include "xform.h"
#include <math.h>

xform::xform() {
  sxo = 0;
  syo = 0;
  sxx = 1;
  sxy = 0;
  syy = 1;
  syx = 0;
  r = 0;
  z = 1;
}

point xform::p_to_s(point p) {
  point s;
  s.x = sxo + p.x * sxx + p.y * sxy;
  s.y = syo + p.x * syx + p.y * syy;
  return s;
}

point xform::p_to_s(double x, double y) {
  point s;
  s.x = sxo + x * sxx + y * sxy;
  s.y = syo + x * syx + y * syy;
  return s;
}

point xform::s_to_p(point s) {
  point p;
  p.x = -(sxo*syy + sxy*(s.y - syo) - syy*s.x) / (sxx*syy - sxy*syx);
  p.y =  (sxo*syx + sxx*(s.y - syo) - syx*s.x) / (sxx*syy - sxy*syx);
  return p;
}

void xform::pan(int delta_sx, int delta_sy) {
  sxo += delta_sx;
  syo += delta_sy;
}

void xform::zoom(double f, int sx_zoom_center, int sy_zoom_center) {
  z *= f;
  sxx *= f;
  sxy *= f;
  syy *= f;
  syx *= f;
  sxo = f * sxo + (1 - f) * sx_zoom_center;
  syo = f * syo + (1 - f) * sy_zoom_center;
}

void xform::rotate(double delta_r, int cent_sx, int cent_sy) {
  const double rpd = M_PI / 180.0;
  double tx1, ty1;  // current origin as measured from cent
  double tx2, ty2;  // new origin as measured from cent
  r += delta_r;
  while (r >= 360) r -= 360;
  while (r < 0)   r += 360;
  double a = r * rpd;
  double delta_a = delta_r * rpd;

  // xform sxo and syo to tx1 and ty1
  tx1 = sxo - cent_sx;
  ty1 = syo - cent_sy;
  // rotate rx and ry
  tx2 =  tx1 * cos(delta_a) + ty1 * sin(delta_a);
  ty2 = -tx1 * sin(delta_a) + ty1 * cos(delta_a);
  // xform rx and ry bac to new sxo and syo
  sxo = tx2 + cent_sx;
  syo = ty2 + cent_sy;
  // update rotation and scaling constants
  sxx = z * cos(a);
  sxy = z * sin(a);
  syx = -z * sin(a);
  syy = z * cos(a);
}

