// xform - A class that does 2D transformations: translate, rotate, pan & zoom
// by Thomas Kisko

#ifndef _xform_
#define _xform_

// a simple point class
class point {
public:
  point() : x(0), y(0) {}
  point(int _x, int _y) : x(_x), y(_y) {}
  int  x;
  int  y;
};

// the xform class
class xform {
private:
  double sxo; // screen x of origin
  double syo; // screen y of origin
  double sxx; // screen x per x
  double sxy; // screen x per y
  double syy; // screen y per y
  double syx; // screen y per x
  double r;   // rotation in degrees
  double z;   // zoom, 2 means double size
public:
  xform();
  xform(double sxo_, double syo_, double sxx_, double sxy_, double syy_, double syx_);
  point p_to_s(point p);   // transform a point p to screen coordinates
  point p_to_s(double x, double y);  // transform (x, y) to screen coodinates
  point s_to_p(point s);   // transform a screen location to a point in our frame of reference
  void zoom(double f, int sx_zoom_center, int sy_zoom_center);  // zoom, f = 2 means double size
  void pan(int delta_sx, int delta_sy);  // pan in any direction
  void rotate(double delta_r, int sx, int sy);  // rotated delta_r degrees, centered at screen coordinates sx, sy
};
#endif

